/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: blog
 * Version: 1.08.00-BETA-00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.blog.processus;

import java.io.IOException;
import java.util.List;

import org.apache.commons.collections4.SetUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.core.LangueUtil;
import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.kosmos.blog.om.Articleblog;
import com.kosmos.blog.om.Blog;
import com.kosmos.blog.util.BlogUtil;
import com.kosmos.blog.util.RubriqueUtil;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.extension.module.plugin.rubrique.PageAccueilRubriqueManager;
import com.kportal.extension.module.plugin.rubrique.impl.BeanFichePageAccueil;
import com.univ.objetspartages.bean.AccueilRubriqueBean;
import com.univ.objetspartages.bean.MediaBean;
import com.univ.objetspartages.bean.RubriqueBean;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.processus.ControleurUniv;
import com.univ.objetspartages.processus.SaisieFiche;
import com.univ.objetspartages.services.ServiceMedia;
import com.univ.objetspartages.services.ServiceRubrique;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.objetspartages.util.MediaUtils;
import com.univ.url.service.ServiceUrl;
import com.univ.utils.json.CodecJSon;

/**
 * Processus de gestion des blogs.
 */
public class SaisieBlog extends SaisieFiche {

    private static final String ECRAN_RECHERCHE = "RECHERCHE";

    private static final String ECRAN_PRINCIPAL = "PRINCIPAL";

    private static final String ECRAN_CONFIRMATION = "CONFIRMATION";

    private static final Logger logger = LoggerFactory.getLogger(SaisieBlog.class);

    private Blog blog = null;

    private final ServiceMedia serviceMedia;

    /**
     * Constructeur.
     *
     * @param infoBean
     */
    public SaisieBlog(final InfoBean infoBean) {
        super(infoBean);
        serviceMedia = ServiceManager.getServiceForBean(MediaBean.class);
    }

    /**
     * Point d'entree du processus.
     */
    @Override
    public boolean traiterAction() throws Exception {
        infoBean.set("CODE_OBJET", "9120");
        blog = new Blog();
        blog.init();
        traiterActionParDefaut(blog);
        infoBean.set("NOM_ONGLET", "blog");
        return etat == FIN;
    }

    /**
     * Affichage de l'ecran des criteres de recherche d'un(e) blog.
     */
    @Override
    protected void preparerRECHERCHE() throws Exception {
        ecranLogique = ECRAN_RECHERCHE;
        infoBean.set("LISTE_LANGUES", LangueUtil.getListeLangues(this.getLocale()));
        infoBean.set("LISTE_THEMATIQUES", LabelUtils.getLabelCombo("04", this.getLocale()));
    }

    /**
     * Affichage de l'ecran de saisie d'un(e) blog.
     */
    @Override
    protected void preparerPRINCIPAL() throws Exception {
        ecranLogique = ECRAN_PRINCIPAL;
        infoBean.set("ID_BLOG", blog.getIdBlog().toString());
        infoBean.set("TITRE", blog.getTitre());
        infoBean.set("DESCRIPTION", blog.getDescription());
        infoBean.set("THEMATIQUE", blog.getThematique());
        infoBean.set("LIBELLE_THEMATIQUE", blog.getLibelleThematique());
        infoBean.set("LISTE_THEMATIQUES", LabelUtils.getLabelCombo("04", LangueUtil.getLocale(blog.getLangue())));
        if (blog.getArchivage() == null || blog.getArchivage().length() == 0) {
            infoBean.set("ARCHIVAGE", "0");
        } else {
            infoBean.set("ARCHIVAGE", blog.getArchivage());
        }
        infoBean.set("CODE_RUBRIQUE", blog.getCodeRubrique());
        final ServiceRubrique serviceRubrique = ServiceManager.getServiceForBean(RubriqueBean.class);
        final RubriqueBean rubriqueBean = serviceRubrique.getRubriqueByCode(blog.getCodeRubrique());
        String rubriqueLabel = StringUtils.EMPTY;
        if (rubriqueBean != null) {
            rubriqueLabel = rubriqueBean.getIntitule();
        }
        infoBean.set("LIBELLE_CODE_RUBRIQUE", rubriqueLabel);
        infoBean.set("CODE_RATTACHEMENT", blog.getCodeRattachement());
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        infoBean.set("LIBELLE_CODE_RATTACHEMENT", serviceStructure.getDisplayableLabel(blog.getCodeRattachement(), blog.getLangue()));
        infoBean.set("LIBELLE_AFFICHABLE", blog.getLibelleAffichable());
        if (infoBean.get("SAISIE_FRONT") == null) {
            infoBean.set("SOUS_ONGLET", "PRINCIPAL"); // onglet par defaut
        }
        final List<Articleblog> billets = blog.getArticles(this);
        if (billets != null && !billets.isEmpty()) {
            infoBean.set("ARTICLES", billets);
        }
        final RubriqueBean rubriqueBlog = serviceRubrique.getRubriqueByCode(blog.getCode());
        if (rubriqueBlog != null && rubriqueBlog.getIdBandeau() != null) {
            infoBean.set("ID_BANDEAU", rubriqueBlog.getIdBandeau().toString());
        }
        infoBean.set("LISTE_BANDEAUX", MediaUtils.getMediasAsMap(serviceMedia.getByTypeAndTypeMedia("photo", "0100")));
        ControleurUniv.preparerPRINCIPAL(infoBean, blog, this);
    }

    @Override
    protected void alimenterDonneesFiche() throws Exception {
        blog.setTitre((String) infoBean.get("TITRE"));
        blog.setDescription((String) infoBean.get("DESCRIPTION"));
        blog.setThematique((String) infoBean.get("THEMATIQUE"));
        blog.setArchivage((String) infoBean.get("ARCHIVAGE"));
        blog.setCodeRubrique((String) infoBean.get("CODE_RUBRIQUE"));
        blog.setCodeRattachement((String) infoBean.get("CODE_RATTACHEMENT"));
    }

    /**
     * Traite l'ecran de saisie d'une fiche.
     */
    @Override
    protected void traiterPRINCIPAL() throws Exception {
        final ServiceRubrique serviceRubrique = ServiceManager.getServiceForBean(RubriqueBean.class);
        // chargement de l'objet
        if (infoBean.getEtatObjet().equals(InfoBean.ETAT_OBJET_CREATION)) {
            blog.init();
        } else {
            blog.setIdBlog(new Long(infoBean.getString("ID_BLOG")));
            blog.retrieve();
        }
        Long idBandeau = null;
        boolean isNewBlog = false;
        String blogBaseSlug = null;
        // changement d'onglet
        if (action.equals(InfoBean.ACTION_ONGLET)) {
            infoBean.set("SOUS_ONGLET", infoBean.getString("SOUS_ONGLET_DEMANDE"));
        }
        // validation de l'ecran
        else if (action.equals(InfoBean.ACTION_ENREGISTRER)) {
            if (StringUtils.isEmpty((String) infoBean.get("CODE_RUBRIQUE"))) {
                throw new ErreurApplicative(BlogUtil.getMessage("BO.BLOG.SAISIE.RUBRIQUE_OBLIGATOIRE"));
            }
            alimenteDonneesCreation(blog, false);
            alimenterDonneesFiche();
            if (infoBean.get("ID_BANDEAU") != null && ((String) infoBean.get("ID_BANDEAU")).length() > 0) {
                idBandeau = Long.valueOf((String) infoBean.get("ID_BANDEAU"));
            }
            if (!"1".equals(infoBean.getString("APERCU")) && (!RubriqueUtil.exists(blog.getCode()) || StringUtils.isBlank(blog.getCode()))) {
                logger.debug("- rubrique de blog absente : création à la volée...");
                isNewBlog = true;
                blogBaseSlug = ServiceUrl.generateSlug(blog.getLibelleAffichable());
                if(StringUtils.isEmpty(infoBean.getString(ServiceUrl.SLUG))){
                    infoBean.set(ServiceUrl.SLUG, blogBaseSlug  + "-blog");
                }
            } else {
                RubriqueBean rubriqueBlog = serviceRubrique.getRubriqueByCode(blog.getCode());
                if (rubriqueBlog != null && idBandeau != null) {
                    rubriqueBlog.setIdBandeau(idBandeau);
                }
            }
            logger.debug("< ACTION_ENREGISTRER : rubrique de blog OK");
        }
        ecranLogique = ControleurUniv.traiterPRINCIPAL(infoBean, blog, this);

        if(isNewBlog){
            serviceRubrique.saveRubriqueAvecPageAccueil(getNewSectionBlogBean(idBandeau, blogBaseSlug), SetUtils.emptySet(), getSectionHomeBean());
            logger.debug("- rubrique de blog créée");
        }

        if (infoBean.get("SAISIE_FRONT") != null) {
            ecranLogique = ECRAN_CONFIRMATION;
        }
        if (ecranLogique.length() == 0) {
            etat = FIN;
        }
    }

    private RubriqueBean getNewSectionBlogBean(Long idBandeau, String blogBaseSlug){
        RubriqueBean rubriqueBlog = new RubriqueBean();
        if (StringUtils.isBlank(blog.getCode())) {
            final String timeStamp = Long.toString(System.currentTimeMillis());
            rubriqueBlog.setCode(timeStamp.substring(timeStamp.length() - 8, timeStamp.length()));
        } else {
            rubriqueBlog.setCode(blog.getCode());
        }
        rubriqueBlog.setIntitule("Blog [" + StringUtils.abbreviate(blog.getTitre(),248) + "]");
        rubriqueBlog.setNomOnglet(StringUtils.abbreviate(blog.getTitre(),64));
        rubriqueBlog.setCategorie("HIDDEN");
        rubriqueBlog.setCodeRubriqueMere(blog.getCodeRubrique());
        rubriqueBlog.setLangue(blog.getLangue());
        if (idBandeau != null) {
            rubriqueBlog.setIdBandeau(idBandeau);
        }

        rubriqueBlog.setSlug(blogBaseSlug);

        return rubriqueBlog;
    }

    private AccueilRubriqueBean getSectionHomeBean() throws IOException{
        final BeanFichePageAccueil beanFichePageTete = new BeanFichePageAccueil();
        beanFichePageTete.setCode(blog.getCode());
        beanFichePageTete.setLangue(blog.getLangue());
        beanFichePageTete.setObjet("blog");
        AccueilRubriqueBean accueilRubriqueBlog = new AccueilRubriqueBean();
        accueilRubriqueBlog.setPageAccueil(CodecJSon.encodeObjectToJSonInString(beanFichePageTete));
        accueilRubriqueBlog.setTypeRubrique(PageAccueilRubriqueManager.getInstance().getTypePageAcceuilParBeanPageAccueil(beanFichePageTete));
        return accueilRubriqueBlog;
    }
}
