/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: blog
 * Version: 1.08.00-BETA-00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.blog.util;

import java.util.Collections;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.kosmos.blog.om.Articleblog;
import com.kosmos.blog.om.Blog;
import com.kportal.core.config.MessageHelper;
import com.kportal.core.config.PropertyHelper;
import com.kportal.extension.ExtensionHelper;
import com.kportal.extension.IExtension;
import com.univ.utils.ContexteUniv;
import com.univ.utils.ContexteUtil;
import com.univ.utils.URLResolver;
import com.univ.utils.UnivWebFmt;

public class BlogUtil {

    /**
     * L'identifiant de l'extension Agenda
     */
    public static final String ID_EXTENSION = "blog";

    public static final String PARAM_MOT_CLE = "KEYWORDS";

    public static final String PARAM_FILTRE_ANNEE = "FILTER_YEAR";

    public static final String PARAM_FILTRE_MOIS = "FILTER_MONTH";

    public static final String PARAM_FILTRE_FROM = "FROM";

    private static final Logger LOG = LoggerFactory.getLogger(BlogUtil.class);

    private static final int NB_RESULTAT_DEFAUT = 5;

    private static final String PROPERTY_NB_RESULTAT = "blog.fiche.nombre.billet";

    /**
     * Récupère la liste des articles du blog courant en fonction des paramètres fourni.
     *
     * @param blog
     *            le blog de la fiche courante
     * @param filtreMotCle
     *            un filtre sur les tags
     * @param filterMois
     *            le filtre sur les mois
     * @param filtreAnnee
     *            le filtre sur l'année
     * @return la liste des artciles de blog.
     */
    public static final List<Articleblog> getArticlesBlogs(final Blog blog, final String filtreMotCle, final String filterMois, final String filtreAnnee) {
        final ContexteUniv ctx = ContexteUtil.getContexteUniv();
        List<Articleblog> listeArticles = null;
        Date dateRef = null;
        try {
            if (StringUtils.isNotBlank(filtreAnnee) && StringUtils.isNumeric(filtreAnnee)) {
                dateRef = new Date(Long.valueOf(filtreAnnee));
                listeArticles = blog.getYearArticles(ctx, dateRef);
            } else if (StringUtils.isNotBlank(filterMois) && StringUtils.isNumeric(filterMois)) {
                dateRef = new Date(Long.valueOf(filterMois));
                listeArticles = blog.getMonthArticles(ctx, dateRef);
            } else if (StringUtils.isNotBlank(filtreMotCle)) {
                listeArticles = blog.getKeywordArticles(ctx, filtreMotCle);
            } else {
                listeArticles = blog.getArticles(ctx);
            }
        } catch (final Exception e) {
            LOG.error("impossible de récupérer les articles du blog courant.", e);
            listeArticles = Collections.<Articleblog>emptyList();
        }
        return listeArticles;
    }

    /**
     * Tronque la liste d'article fourni en paramètre en fonction de ce qui doit être affiché dans la page.
     *
     * @param listeArticles
     *            la liste des articles à tronquer
     * @param from
     *            l'index de début à partir du quel on tronque la liste
     * @return la liste d'artcle tronqué à afficher dans la page courante.
     */
    public static final List<Articleblog> getArticlesBlogsAParcourir(final List<Articleblog> listeArticles, final String from) {
        final int debutFenetre = getDebutPageCourante(from, listeArticles.size());
        final int finFenetre = getFinPageCourante(from, listeArticles.size());
        return listeArticles.subList(debutFenetre, finFenetre);
    }

    /**
     *
     * @return le nombre d'article à afficher par page
     */
    public static final int getNombreResultatParPage() {
        int nbResultatsParPage = NB_RESULTAT_DEFAUT;
        final String proprieteNbBillet = PropertyHelper.getProperty(ID_EXTENSION, PROPERTY_NB_RESULTAT);
        if (StringUtils.isNotBlank(proprieteNbBillet) && StringUtils.isNumeric(proprieteNbBillet)) {
            nbResultatsParPage = Integer.valueOf(proprieteNbBillet);
        }
        return nbResultatsParPage;
    }

    /**
     * Calcule l'index de la page courante. A partir de quel index on affiche les articles du blog
     *
     * @param parametreFrom
     *            le paramètre de la requete
     * @param nbResultats
     *            le nombre de résultat total
     * @return l'index du premier article à afficher
     */
    public static final int getDebutPageCourante(final String parametreFrom, final int nbResultats) {
        int debutFenetre = 0;
        if (StringUtils.isNotEmpty(parametreFrom) && StringUtils.isNumeric(parametreFrom)) {
            debutFenetre = Integer.valueOf(parametreFrom);
        }
        if (debutFenetre > nbResultats) {
            debutFenetre = nbResultats;
        }
        return debutFenetre;
    }

    /**
     * Calcule l'index de fin de la page courante
     *
     * @param parametreFrom
     *            le paramètre de la requete
     * @param nbResultats
     *            le nombre de résultat total
     * @return l'index du dernier article à afficher
     */
    public static final int getFinPageCourante(final String parametreFrom, final int nbResultats) {
        final int debutFenetre = getDebutPageCourante(parametreFrom, nbResultats);
        final int nbResultatsParPage = BlogUtil.getNombreResultatParPage();
        int finFenetre = debutFenetre + nbResultatsParPage;
        if (nbResultats < finFenetre) {
            finFenetre = nbResultats;
        }
        return finFenetre;
    }

    /**
     * Calcule le nombre total de page à afficher
     *
     * @param nbResultats
     * @return
     */
    public static final int getNombreTotalPage(final int nbResultats) {
        final int nombreResultatParPage = getNombreResultatParPage();
        int nbTotalPages = nbResultats / nombreResultatParPage;
        if (nbResultats % nombreResultatParPage > 0) {
            nbTotalPages++;
        }
        return nbTotalPages;
    }

    /**
     * Construit l'url de navigation des articles de blog
     *
     * @param blogCourant
     * @param parametreFrom
     * @param filtreMotCle
     * @param filterMois
     * @param filtreAnnee
     * @return
     */
    public static final String getUrlPageBlog(final Blog blogCourant, final int parametreFrom, final String filtreMotCle, final String filterMois, final String filtreAnnee) {
        final ContexteUniv ctx = ContexteUtil.getContexteUniv();
        final StringBuilder urlFiche = new StringBuilder();
        try {
            urlFiche.append(URLResolver.getAbsoluteUrl(UnivWebFmt.determinerUrlFiche(ctx, blogCourant), ctx));
            if (StringUtils.isNotBlank(filtreAnnee)) {
                urlFiche.append("&").append(PARAM_FILTRE_ANNEE).append("=").append(filtreAnnee);
            }
            if (StringUtils.isNotBlank(filterMois)) {
                urlFiche.append("&").append(PARAM_FILTRE_MOIS).append("=").append(filterMois);
            }
            if (StringUtils.isNotBlank(filtreMotCle)) {
                urlFiche.append("&").append(PARAM_MOT_CLE).append("=").append(filtreMotCle);
            }
            urlFiche.append("&").append(PARAM_FILTRE_FROM).append("=").append(parametreFrom);
        } catch (final Exception e) {
            LOG.error("Impossible de récupérer l'url du blog courant", e);
        }
        return urlFiche.toString();
    }

    public static IExtension getExtension() {
        return ExtensionHelper.getExtensionManager().getExtension(ID_EXTENSION);
    }

    public static String getMessage(final String idMessage) {
        return MessageHelper.getMessage(ID_EXTENSION, idMessage);
    }
}
